/*
**	RAVE3DDemo.c 0.5 (16-Feb-1999)
** 
**	This is an example program for cgx3drave.library
**
**      Written by Frank Mariak
**
**      Command line options:
**      OBJTYPE/N,RUNMODE/S,LIGHTING/S,TEXTURING/S,DEPTH/N,TRANSPARENCY/S
**
**      OBJTYPE/N       selects between object type 1 (ring), 2 (cylinder) or
**                      3 (cube)
**	TEXNUM/N	selects the texture that should be used for texturing
**			0 (Fans.bmp), 1 (FloorTile.bmp), 2 (SonicWall.bmp),
**			3 (SteelDoor.bmp) [default], 4 (WarningWall.bmp) or 
**			5 (WarningWall2.bmp)
**      RUNMODE/S       makes the program render as many frames as possible
**                      and outputs statistics at the end
**      LIGHTING/S      enable lighting
**      TEXTURING/S     enable texturing
**      DEPTH/S         specify depth for rendering screen (15[default], 16 
**			or 32)
**      TRANSPARENCY/S  enable transparency (only object 3)
**      FILTERING/S     enable bilinear filtering if texturing is enabled
**
**	Textures are "borrowed" from  the  source  code  distribution  of
**	Alien  Breed  3
**
**	If you have OS3.1 and lowlevel.library installed, you can use a CD32 
**	compatible controller in port 1 to move/rotate object
**
**	Copyright  1996-1999 by Vision Factory Development
**      All Rights reserved.
**
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include <cybergraphx/cybergraphics.h>
#include <cybergraphx/cgx3drave.h>
#include <dos/dos.h>
#include <exec/execbase.h>
#include <exec/memory.h>
#include <intuition/intuition.h>
#include <libraries/lowlevel.h>

#include <devices/timer.h>

#include <proto/cybergraphics.h>
#include <proto/cgx3drave.h>
#include <proto/dos.h>
#include <proto/exec.h>
#include <proto/graphics.h>
#include <proto/intuition.h>
#include <proto/lowlevel.h>
#include <proto/timer.h>


typedef struct
{
        UBYTE ac,rc,gc,bc;
} ARGB32;

typedef union
{
        LONG    argbval;
        ARGB32  argb32;
} ARGB;

typedef struct
{
        long x,y,z;
	long u,v;
        ARGB  color;
	float	a,r,g,b;
} Point3D;


#define DISPLAYWIDTH    640
#define DISPLAYHEIGHT   480
#define WINX	DISPLAYWIDTH
#define WINY	DISPLAYHEIGHT
#define FOCUS 0.0122 //

/* 0x7f = 50%, 0xff = 0%, 0x00 = 100%  - used for transparency option, 0x7f recommended */
#define OPACITY_LEVEL 0x7f

#define Up      0x4c
#define Left    0x4f
#define Down    0x4d
#define Right   0x4e

#define AUp     0x3e
#define ALeft   0x2d
#define ADown   0x1e
#define ARight  0x2f
#define AAxUp   0x1d
#define AAxDown 0x3f
#define Plus    0x5e
#define Minus   0x4a
#define Help    0x5f
#define esc     0x45

#define TEXMAPSIZE      128*128*3

ULONG			ticks,TDevFlag = 0;
ULONG			allticks,secs,mics;
float			micros;
struct MsgPort		*timermp=0;
struct timerequest	*timerio=0;
struct EClockVal	MyOldEClockVal;
struct EClockVal	MyEClockVal;
struct Library		*TimerBase=0;

long    ZBufFlag        = 0;
long    TextureFlag     = 0;
long    LightingFlag    = 0;
long    Depth	        = 0;
long    BlendFlag       = 0;
long	FilterFlag	= 0;
long	InfoFlag	= 0;

void RenderWorld(TQAVTexture *,int);
void TransWorld(Point3D *,TQAVTexture *,
                int,float,float,float,float);

UBYTE   *VString = "\0$VER: RAVE3DDemo.c 0.5 (16-02-99)  1996-1999 by Vision Factory Development, written by F.Mariak";

STRPTR	texnames[] =
	{
	 (STRPTR)"gfx/Fans.bmp",
	 (STRPTR)"gfx/FloorTile.bmp",
	 (STRPTR)"gfx/SonicWall.bmp",
	 (STRPTR)"gfx/SteelDoor.bmp",
	 (STRPTR)"gfx/WarningWall.bmp",
	 (STRPTR)"gfx/WarningWall2.bmp"
	};

Point3D		ringdata[] =
		{
#include	"obj_ring.cdat"
		};

Point3D		cubedata[] =
		{
#include	"obj_cube.cdat"
		};

Point3D		cylinderdata[] =
		{
#include        "obj_cylinder.cdat"
		};

#define	NUMCUBES	27

Point3D		mcubedata[(sizeof(cubedata)/sizeof(Point3D))*NUMCUBES];

Point3D	      *polydata=cubedata;
TQAVTexture     *transdata=0;

long            vertexcnt=0;

struct MsgPort *eventmp=0;

struct  IntuitionBase   *IntuitionBase = NULL;
struct  GfxBase         *GfxBase = NULL;
struct  Library         *CyberGfxBase = NULL;
struct  Library         *CGX3DRAVEBase=0;
struct  Library         *LowLevelBase=0;
struct	Screen          *v3d=0;
struct	ScreenBuffer	*sbuf[2];
TQABitmap	*bitmap=0L;
// TQABitmap	*texbitmap=0L;

UBYTE           BufNum=0;
APTR            TexelMap=0;

BOOL	SafeToWrite[2];
BOOL	SafeToChange[2];

     TQADrawContext *drawContext;

struct	TextAttr	topazattr =
	{
	 "topaz.font",
	 8,
	 0,
	 0
	};

struct	TextFont *textfont;

#define RDARG_OBJTYPE   0
#define RDARG_TEXNUM    1
#define RDARG_RUNMODE   2
#define RDARG_LIGHTING  3
#define RDARG_TEXTURING 4
#define RDARG_DEPTH  5
#define RDARG_TRANSPARENCY 6
#define RDARG_FILTERING 7

#define D(x)	;

long	MonitorID=12;

struct Screen *Open3DView(ULONG NomWidth,ULONG NomHeight, ULONG NomDepth)
{
  ULONG DefDispID;
  struct ViewPort *vp;
  struct Screen *scr;
  struct Window *wnd;

  DefDispID=BestCModeIDTags( CYBRBIDTG_MonitorID, MonitorID,
                             CYBRBIDTG_Depth, NomDepth,
                             CYBRBIDTG_NominalHeight, NomHeight,
                             CYBRBIDTG_NominalWidth, NomWidth,
                             TAG_DONE);

  if((DefDispID == INVALID_ID) || (DefDispID == 0))
  {
   D(kprintf("Unable to get valid id\n"));
   return(0L);
  }

  if(scr=OpenScreenTags(0L,  SA_Width,NomWidth,
                             SA_Height,NomHeight,
                             SA_DisplayID,DefDispID,
                             SA_Depth,NomDepth,
                             SA_Draggable, FALSE,
                             TAG_DONE))
  {
   vp=&(scr)->ViewPort;
 
   if(GetVPModeID(vp) != DefDispID)
   {      
    D(kprintf("ERROR: Invalid screen mode ID\n"));
    CloseScreen(scr);
    return(0L);
   }

   sbuf[0] = AllocScreenBuffer(scr, 0L, SB_SCREEN_BITMAP);
   sbuf[1] = AllocScreenBuffer(scr, 0L, SB_COPY_BITMAP);

//   SafeToChange[0] = SafeToChange[1] = TRUE;
//   SafeToWrite[0] = SafeToWrite[1] = TRUE;

   if((sbuf[0] == NULL) || (sbuf[1] == NULL))
   {
    if(sbuf[0]) FreeScreenBuffer(scr,sbuf[0]);
    if(sbuf[1]) FreeScreenBuffer(scr,sbuf[1]);
    sbuf[0]=sbuf[1]=0;
    CloseScreen(scr);
    return(0L);
   }

#if 0
   sbuf[0]->sb_DBufInfo->dbi_SafeMessage.mn_ReplyPort = CreateMsgPort();
   sbuf[1]->sb_DBufInfo->dbi_SafeMessage.mn_ReplyPort = CreateMsgPort();

   sbuf[0]->sb_DBufInfo->dbi_DispMessage.mn_ReplyPort = CreateMsgPort();
   sbuf[1]->sb_DBufInfo->dbi_DispMessage.mn_ReplyPort = CreateMsgPort();
#endif

   SetRGB32(vp,0,0,0,0);

   if(!(wnd = OpenWindowTags(0,
                            WA_Width, NomWidth,
                            WA_Height,NomHeight,
                            WA_Title,(UBYTE *)"Rave3D Window",
                            WA_CustomScreen,scr,
                            WA_IDCMP,0L,
                            WA_Activate,TRUE,
                            WA_DragBar,TRUE,
                            WA_SimpleRefresh,TRUE,
                            WA_Backdrop,TRUE,
                            WA_Borderless,TRUE,
                           TAG_DONE)
                ))
   {
    D(kprintf("ERROR: Couldn't open any Window\n"));
    if(sbuf[0]) FreeScreenBuffer(scr,sbuf[0]);
    if(sbuf[1]) FreeScreenBuffer(scr,sbuf[1]);
    sbuf[0]=sbuf[1]=0;
    CloseScreen(scr);
    return(0L);
   }

   return(scr);
  }
  else
  {
    D(kprintf("ERROR: No screen\n"));
    return(0L);
  }
}



void	main(int argc,char *argv[])
{
  float wz=0.0,wx=0.0,wy=0.0;
  int	gz = 2000;
  struct  ExecBase *ExecBase=(struct ExecBase *)*((ULONG *)4L);
  BOOL Update=FALSE,RunFlag=TRUE,Transform=FALSE;
  struct IntuiMessage *msg = NULL;
  long    frames=0,objtype=0,runmode,texnum=3;
  LONG         rdargs[8];
  struct   RDArgs *rda=0;
  UBYTE *rdatemplate=(STRPTR)"OBJTYPE/N,TEXNUM/N,RUNMODE/S,LIGHTING/S,TEXTURING/S,DEPTH/N,TRANSPARENCY/S,FILTERING/S";

  rdargs[0]=0;
  rdargs[1]=0;
  rdargs[2]=0;
  rdargs[3]=0;
  rdargs[4]=0;
  rdargs[5]=0;
  rdargs[6]=0;
  rdargs[7]=0;

#ifdef _M68881
  if(!(ExecBase->AttnFlags & AFF_68881))
  {
   printf("You need at least a 68881 floating point unit !\n");
   return;
  }
#endif

  if(rda=ReadArgs(rdatemplate,rdargs,0L))
  {
   if ((timermp = CreatePort(NULL, 0)) != NULL)
   {
    if ((timerio = (struct timerequest*) CreateStdIO(timermp)) != NULL)
    {
     if (OpenDevice("timer.device", UNIT_VBLANK, (struct IORequest *)timerio, 0) == 0L)
     {
      TimerBase	=	(struct Library*) timerio->tr_node.io_Device;

      if (IntuitionBase=(struct IntuitionBase *)OpenLibrary("intuition.library",39L))
      {
       if (GfxBase=(struct GfxBase *)OpenLibrary("graphics.library",39L))
       {

        if(CyberGfxBase=(struct Library *)OpenLibrary("cybergraphics.library",41L))
	{

         if(CGX3DRAVEBase=(struct Library *)OpenLibrary("cgx3drave.library",0L))
         {
	  long i;
	  struct Screen *pubscr;

	  if(pubscr=LockPubScreen(0L))
	  {
	   MonitorID = ((GetVPModeID(&pubscr->ViewPort)&0x00f00000))>>20;
	   UnlockPubScreen(0L,pubscr);
	  }

#if 0
          if(LowLevelBase = OpenLibrary("lowlevel.library",40L))
          {
           SetJoyPortAttrs(1L,
                           SJA_Type, SJA_TYPE_GAMECTLR,
                           TAG_DONE);
          }
#endif
          runmode = rdargs[RDARG_RUNMODE];

          if(rdargs[RDARG_DEPTH])
          {
		ULONG depth = *((ULONG *)rdargs[RDARG_DEPTH]);

	   switch(depth)
	   {
		case 15:
		case 16:
		case 32:
			Depth = depth;
			break;
		default:
			Depth = 15;
			break;
	   }
          }
	  else
	  {
		Depth = 15;
	  }

          if(rdargs[RDARG_TEXTURING])
          {
           TextureFlag     = 1;
          }

          if(rdargs[RDARG_LIGHTING])
          {
           LightingFlag    = 1;
          }

          if(rdargs[RDARG_TRANSPARENCY])
          {
           BlendFlag    = 1;
          }

          if(rdargs[RDARG_FILTERING])
          {
           FilterFlag    = 1;
          }

          if(rdargs[RDARG_OBJTYPE])
          {
           objtype = *(ULONG *)rdargs[RDARG_OBJTYPE];
          }

          switch(objtype)
          {
           case 2:
                polydata        = cylinderdata;
                vertexcnt     = ((sizeof(cylinderdata)/(sizeof(Point3D))));
                BlendFlag = 0;
                break;
           case 3:
                polydata        = cubedata;
                vertexcnt     = ((sizeof(cubedata)/(sizeof(Point3D))));
                TextureFlag = 1;
                ZBufFlag        = 1;    
                break;
           case 4:
                polydata        = mcubedata;

		for(i=0;i<NUMCUBES;i++)
		{
		 CopyMem(cubedata,&mcubedata[i*(sizeof(cubedata)/sizeof(Point3D))],sizeof(cubedata));
		}
                vertexcnt     = ((sizeof(mcubedata)/(sizeof(Point3D))));
//                TextureFlag = 0;
		BlendFlag = 0;
                ZBufFlag        = 1;    
                break;
           default:
                objtype         = 1;
                polydata        = ringdata;
                vertexcnt     = ((sizeof(ringdata)/(sizeof(Point3D))));
                TextureFlag = 1;
                BlendFlag = 0;
                ZBufFlag        = 1;    
                break;
          }


          if(rdargs[RDARG_TEXNUM])
          {
           texnum = *(ULONG *)rdargs[RDARG_TEXNUM];
          }

	  switch(texnum)
	  {
		case 0:
		case 1:
		case 2:
		case 3:
		case 4:
		case 5:
			break;
		default:
			texnum = 3;

	  }


	  if(objtype == 4)
	{
	 long i,j,k,n;

	 n=vertexcnt / NUMCUBES;

	 for(i=0;i<NUMCUBES;i++)
		{
			for(j=0;j<n;j++)
			{
			 polydata[i*n + j].x /= 5;
			 polydata[i*n + j].y /= 5;
			 polydata[i*n + j].z /= 5;

			 polydata[i*n + j].x += (((i%9)%3)-1)*103;
			 polydata[i*n + j].y += (((i%9)/3)-1)*103;
			 polydata[i*n + j].z += ((i/9)-1)*103;

			polydata[i*n + j].color.argbval = 0xff000000;

				 switch(j%6)
				 {
				  case 0:
					k= 0xff;
					break;
				  case 1:
				  case 2:
				  case 3:
				  case 4:
					k= 0x9f;
					break;
				  case 5:
					k= 0x4f;
					break;
				 }

			 switch(j/6)
			 {
			  case 0:	// front
				if((i / 9) == 0)
				{
				 polydata[i*n + j].color.argbval |= (k<<16);
				}
				break;
			  case 1:	// back
				if((i / 9) == 2)
				{
				 polydata[i*n + j].color.argbval |= (k<<8);
				}
				break;
			  case 2:	// left
				if((i % 3) == 0)
				{
				 polydata[i*n + j].color.argbval |= (k);
				}
				break;
			  case 3:	// right
				if((i % 3) == 2)
				{
				 polydata[i*n + j].color.argbval |= (k | (k<<8));
				}
				break;
			  case 4:	// top
				if((i % 9) < 3)
				{
				 polydata[i*n + j].color.argbval |= (k | (k<<16));
				}
				break;
			  case 5:	// bottom
				if((i % 9) >= 6)
				{
				 polydata[i*n + j].color.argbval |= ((k<<16)|(k<<8));
				}
				break;
			  default:
				break; 
			 }
			}
		}
	}

          for (i=0;i<vertexcnt;i++)
          {
                        polydata[i].color.argb32.ac = BlendFlag ? OPACITY_LEVEL : LightingFlag ? OPACITY_LEVEL : 0xff;
			polydata[i].a = (1.0f/255) * (polydata[i].color.argb32.ac);
			polydata[i].r = (1.0f/255) * (polydata[i].color.argb32.rc);
			polydata[i].g = (1.0f/255) * (polydata[i].color.argb32.gc);
			polydata[i].b = (1.0f/255) * (polydata[i].color.argb32.bc);
          }

          if(TextureFlag)
          {
#define TEXSIZE	128

           if(TexelMap = AllocVec(TEXSIZE*TEXSIZE*4 + 10000,MEMF_CLEAR))
           {
            BPTR fh;
            long i;

  	    if(fh=Open(texnames[texnum],MODE_OLDFILE))
 	    {
	     UBYTE *texsource = TexelMap;
	     ULONG *texdest = TexelMap;
	     ULONG header = 0;
	     Read(fh,&header,4L);

		// 0 'BM'
		// 2.L file size
		// 10.L headersize
		
		// 0.W infoheader size
		// 4.L width
		// 8.L height
		// 12.W Planes (always 1)
		// 14.W bits per pixel
		// 24.L density x
		// 28.L density x
		// 32.L colormap entries

	     if(header == 0x424d36c0)	// we only support 128x128x3 RGB BMPs
	     {
		long x,y,val;
		UWORD	*adr1;

	      Seek(fh,54L,OFFSET_BEGINNING);

			// bmp stores scanlines bottom-to-top, so we invert
		for(y=TEXSIZE-1;y>=0;y--)
		{
		      Read(fh,(void *)((long)TexelMap+(TEXSIZE * 3 * y)),TEXSIZE*3);
		}

	      i = TEXSIZE*TEXSIZE - 1;

		#define ARGB2RGB16(ARGB)        (((UWORD)(ARGB>>8)&0xf800)|((UWORD)(ARGB>>5)&0x07E0)|((UWORD)(ARGB>>3)&0x1f))

              while(i--)
              {
	       texdest[i] = 0xff000000|(texsource[i*3 + 2]<<16) | (texsource[i*3 + 1]<<8) | (texsource[i*3 + 0]<<0);
              }
             }
	     else
	     {
              printf("Invalid texture datafile %s !\n",texnames[texnum]);
             }
 	     Close(fh);
 	    }
            else
            {
             printf("Unable to load texture datafile %s !\n",texnames[texnum]);
            }
 
            switch(objtype)
            {
                case 2:
                	  for (i=0;i<vertexcnt;i++)
                 	  {
                           polydata[i].u *= 2;
                           polydata[i].v *= 4;
                          }
                        break;
		case 1:
                case 3:
		default:
                        break;
            }

           }
           else
           {
            printf("Unable to get memory for texelmap !\n");
            TextureFlag = 0;
	    FilterFlag = 0;
           }
          }

        
            //
            // now allocate a buffer for our transformed vertices to speed
            // up rendering when nothing has moved

          if(transdata=(TQAVTexture *)AllocVec(sizeof(TQAVTexture)*vertexcnt + 10000,MEMF_CLEAR))
          {

            if(v3d=Open3DView(DISPLAYWIDTH,
                              DISPLAYHEIGHT,
                              Depth))
            {
             if(eventmp=CreateMsgPort())
             {
	   TQAEngine *engine;

	   void *GHandle;
	   struct BitMap *bm=v3d->RastPort.BitMap;

	      v3d->FirstWindow->UserPort = eventmp;

              ModifyIDCMP(v3d->FirstWindow,IDCMP_RAWKEY);


   if(GHandle = CreateGHandleTags(CGXR3DT_FrontMap, sbuf[0]->sb_BitMap,
				  CGXR3DT_BackMap, sbuf[1]->sb_BitMap,
				  TAG_DONE))
   {
    TQADevice qadev;

    qadev.deviceType = kQADeviceGHandle;
    qadev.device.gHandle = GHandle;

    if(engine=QADeviceGetFirstEngine(&qadev))
    {
		TQARect	rect;
	   long width=0,height=0;
	   width  = GetBitMapAttr(bm,BMA_WIDTH);
	   height = GetBitMapAttr(bm,BMA_HEIGHT);

	     rect.left = 0;
	     rect.top  = 0;
	     rect.right = rect.left + width;
	     rect.bottom = rect.top + height;
	  

     if(QADrawContextNew(&qadev,
			 &rect,
			 0L,
			 engine,
			 kQAContext_DoubleBuffer | (ZBufFlag ? kQAContext_None : kQAContext_NoZBuffer),
			 &drawContext) == kQANoErr)
	{
	     TQATexture	*texture=0;
	     TQAImage	teximage;
	     TQAImage	bmimage;
		UBYTE	strbuf[80];
		struct	BitMap	*txtbm=0L;
		struct	RastPort rp;
		UBYTE	*text="Testtext";

	     QASetFloat(drawContext,kQATag_ColorBG_a, 1.0f);
    	     QASetFloat(drawContext,kQATag_ColorBG_r, 0.01f);
	     QASetFloat(drawContext,kQATag_ColorBG_g, 0.01f);
	     QASetFloat(drawContext,kQATag_ColorBG_b, 0.01f);

		bmimage.width = 320;
		bmimage.height = 100;
		
#define BMF_SPECIALFMT	0x80
// BMF_SPECIALFMT|BMF_DISPLAYABLE|BMF_CLEAR|(PIXFMT_RGB16<<24)

	if(txtbm = AllocBitMap(320,100, 1, BMF_CLEAR , 0L))
	{

		bmimage.rowBytes =  320>>3;	// GetBitMapAttr(txtbm,BMA_WIDTH)*2;
	}

		InitRastPort(&rp);
		rp.BitMap = txtbm;

		textfont = OpenFont(&topazattr);

		if(textfont)
		{
//		 SetFont(&rp,textfont);

		Move(&rp,10,9);
		sprintf(strbuf,"object %ld uses %ld triangles",objtype,vertexcnt/3);
		Text(&rp,strbuf,strlen(strbuf));
		Move(&rp,10,18);
		sprintf(strbuf,"display depth = %ldbit",Depth);
		Text(&rp,strbuf,strlen(strbuf));
		Move(&rp,10,27);
		sprintf(strbuf,"z-buffering %s",ZBufFlag ? "enabled" : "off");
		Text(&rp,strbuf,strlen(strbuf));
		sprintf(strbuf,"texturing %s",TextureFlag ? "enabled" : "off");
		Move(&rp,10,36);
		Text(&rp,strbuf,strlen(strbuf));
		sprintf(strbuf,"lighting %s",LightingFlag ? "enabled" : "off");
		Move(&rp,10,45);
		Text(&rp,strbuf,strlen(strbuf));
		sprintf(strbuf,"transparency %s",BlendFlag ? "enabled" : "off");
		Move(&rp,10,54);
		Text(&rp,strbuf,strlen(strbuf));
		sprintf(strbuf,"filtering %s",FilterFlag ? "enabled" : "off");
		Move(&rp,10,63);
		Text(&rp,strbuf,strlen(strbuf));
		}

		if(txtbm)
		{
		 bmimage.pixmap = txtbm->Planes[0];
#if 1
		 QABitmapNew(engine,
			0L,
			kQAPixel_Alpha1,
			&bmimage,
			&bitmap);
#endif
		}

              if(TextureFlag)
              {
	       teximage.rowBytes = TEXSIZE*4;
	       teximage.width = TEXSIZE;
	       teximage.height = TEXSIZE;
	       teximage.pixmap = TexelMap;

		if(LightingFlag)
		{
		 ULONG	*texdata=TexelMap;
		 for(i=0;i<TEXSIZE*TEXSIZE;i++)  *texdata++ = (*texdata & (0x00ffffff))|(OPACITY_LEVEL << 24);
		}

#if 0
	       QABitmapNew(engine,
				kQATexture_None,
				kQAPixel_RGB32,
				&teximage,
				&texbitmap);
#endif
	       if(QATextureNew(engine,
				kQATexture_None,
				BlendFlag ? kQAPixel_ARGB32 : kQAPixel_ARGB32,
				&teximage,
				&texture) == kQANoErr)
		{
		  QASetInt(drawContext, kQATag_Blend, kQABlend_PreMultiply);
		  QASetInt(drawContext, kQATag_TextureOp, LightingFlag ? kQATextureOp_Decal : kQATextureOp_None);
		  QASetInt(drawContext, kQATag_TextureFilter, FilterFlag ? kQATextureFilter_Best : kQATextureFilter_Fast);
		  QASetInt(drawContext, kQATag_PerspectiveZ, kQAPerspectiveZ_Off);

		  QASetPtr(drawContext, kQATag_Texture, texture);


		}
		else
		{
        	       printf("Unable to create texture handle !\n");
	               TextureFlag = 0L;
		}
             }
          
             if(runmode) ticks=ReadEClock(&MyOldEClockVal);

    	     TransWorld(polydata,transdata,vertexcnt,wx,wy,wz,(float)(1.0f/32767.0f * gz));
    	     RenderWorld(transdata,vertexcnt);
             frames++;

	     while(RunFlag)
	     {
              if(!rdargs[RDARG_RUNMODE])    Wait(1L<<eventmp->mp_SigBit);
              else                          Update = TRUE;

              if(LowLevelBase)
              {
                       ULONG state;

                       state=ReadJoyPort(1L);

                       if(state & JP_TYPE_GAMECTLR)
                        {
                         if(state & (JP_BUTTON_MASK|JP_DIRECTION_MASK))
                         {

                         if(state & JPF_JOY_UP)
                         {
                          wx+= 3.14/18;
                          Update = TRUE;
                          Transform = TRUE;
                         }

                         if(state & JPF_JOY_DOWN)
                         {
                          wx-= 3.14/18;
                          Update = TRUE;
                          Transform = TRUE;
                         }

                         if(state & JPF_JOY_LEFT)
                         {
                          wy-= 3.14/18;
                          Update = TRUE;
                          Transform = TRUE;
                         }

                         if(state & JPF_JOY_RIGHT)
                         {
                          wy+= 3.14/18;
                          Update = TRUE;
                          Transform = TRUE;
                         }

                         if(state & JPF_BUTTON_FORWARD)
                         {
                          if( gz > 600)
                          { gz-= 50;
                            Update = TRUE;
                            Transform = TRUE;
                          }
                         }

                         if(state & JPF_BUTTON_REVERSE)
                         {
                          gz+= 50;
                          Update = TRUE;
                          Transform = TRUE;
                         }


                         if(state & JPF_BUTTON_GREEN)
                         {
                          wz+= 3.14/18;
                          Update = TRUE;
                          Transform = TRUE;
                         }

                         if(state & JPF_BUTTON_YELLOW)
                         {
                          wz-= 3.14/18;
                          Update = TRUE;
                          Transform = TRUE;
                         }
                         }
                        }
                        
                      }

              while(msg=(struct IntuiMessage *)GetMsg(eventmp))
              {
               switch(msg->Class)
               {
                case IDCMP_INTUITICKS:
                     {
                     }
                     break;
                case IDCMP_RAWKEY:
                     {
                        int Code=msg->Code;

                        if(!(Code & 0x80))
                        {
                         Code&=0x7f;
                       switch(Code)
                       {
                         case Up   : wx+= 3.14/36; Update=TRUE; Transform = TRUE; break;
                         case Down : wx-= 3.14/36; Update=TRUE; Transform = TRUE; break;
                         case Right: wy+= 3.14/36; Update=TRUE; Transform = TRUE; break;
                         case Left : wy-= 3.14/36; Update=TRUE; Transform = TRUE; break;
                         case AUp   : wx+= 3.14/18; Update=TRUE; Transform = TRUE; break;
                         case ADown : wx-= 3.14/18; Update=TRUE; Transform = TRUE; break;
                         case ARight: wy+= 3.14/18; Update=TRUE; Transform = TRUE; break;
                         case ALeft : wy-= 3.14/18; Update=TRUE; Transform = TRUE; break;
                         case AAxUp   : wz+= 3.14/18; Update=TRUE; Transform = TRUE; break;
                         case AAxDown : wz-= 3.14/18; Update=TRUE; Transform = TRUE; break;
                         case Plus   : if( gz > 600)
                                        { gz-= 50; Update=TRUE; Transform = TRUE; }
                                       break;
                         case Minus  : gz+= 50; Update=TRUE; Transform = TRUE; break;
        
                         case esc:   RunFlag = FALSE;
					break;
			 case 0x17:	// i
					InfoFlag = !InfoFlag;
					Update=TRUE;
					break;
			 default:
//				printf("Code %lx\n",Code);
				break;
                       }
                     }
                     break;
               };

              }
              ReplyMsg((struct Message *)msg);

     	    }

		if(runmode)
		{
		 wx += 3.14/36;
//		 wy += 3.14/36;
		 wz += 3.14/36;
		 Transform=TRUE;
		}

              if(Transform==TRUE)
              {
               TransWorld(polydata,transdata,vertexcnt,wx,wy,wz,(1.0f/32767.0f) * gz);
              }

              if(Update==TRUE)
              {
               RenderWorld(transdata,vertexcnt);
               frames++;
              }
 
              Update=FALSE;
              Transform=FALSE;
             }

             if(runmode)
             {
              ticks=ReadEClock(&MyEClockVal);

              if (MyEClockVal.ev_hi <= MyOldEClockVal.ev_hi)
              {
               allticks=MyEClockVal.ev_lo - MyOldEClockVal.ev_lo;
              }
              else
              {
               if (MyEClockVal.ev_lo <= MyOldEClockVal.ev_lo)
               {
                allticks=MyEClockVal.ev_lo - MyOldEClockVal.ev_lo;
               }
               else
               {
                allticks=MyOldEClockVal.ev_lo - MyEClockVal.ev_lo;
               }
               allticks+=(MyEClockVal.ev_hi - MyOldEClockVal.ev_hi)<<16;
              }

              secs	=	allticks/ticks;
              micros	=	((float) 1/((float) ticks/ (float) ((allticks-(secs*ticks)))))*1000000;
              mics	=	micros;

              printf("frames rendered %ld,  runtime %ld.%04ld secs\n",frames,secs,mics);
              printf("frames/sec %f\n",(float)(frames*1000)/(float)((secs*1000)+(mics/1000)));
             }

		 if(bitmap) QABitmapDelete(engine,bitmap);

//		 if(texbitmap) QABitmapDelete(engine,texbitmap);

		 if(texture) QATextureDelete(engine,texture);

		if(textfont) CloseFont(textfont);

		if(txtbm) FreeBitMap(txtbm);

		 QADrawContextDelete(drawContext);
		}
		else
		{
		 printf("Unable to create draw context\n");
		}

		}
		else
		{
		 printf("Unable to find drawing engine\n");
		}
	    DeleteGHandle(GHandle);

	}
	else
	{
	 printf("Unable to create GHandle\n");
	}
		v3d->FirstWindow->UserPort = 0L;
             CloseWindow(v3d->FirstWindow);
              DeleteMsgPort(eventmp);
             } // if(V3D_CreateEventPort
             else
             {
              printf("ERROR: Unable to create event port for idcmp messages !\n");
             }

		   if(sbuf[0]) FreeScreenBuffer(v3d,sbuf[0]);
		   if(sbuf[1]) FreeScreenBuffer(v3d,sbuf[1]);
		   sbuf[0]=sbuf[1]=0;
             CloseScreen(v3d);

            } // if(V3D_OpenView
            else
            {
             printf("Unable to open a 3D view !\n");
            }
          }
          else
          { // if(transdata
           printf("Unable to allocate buffer for transformed 3d triangles!\n");
          }

          if(transdata) FreeVec(transdata);
          if(TexelMap) FreeVec(TexelMap);
          if (LowLevelBase) CloseLibrary ((struct Library *)LowLevelBase);

          CloseLibrary((struct Library *)CGX3DRAVEBase);
         } // OpenLib(cgx3drave
         else
         {
          printf ("ERROR: Couldn't open cgx3drave.library v0\n");
         }

         CloseLibrary((struct Library *)CyberGfxBase);
        } // OpenLib(cybergraphics
        else
        {
         printf ("ERROR: Couldn't open cybergraphics.library v41\n");
        }

        CloseLibrary ((struct Library *)GfxBase);
       } // OpenLib(graphics
       else
       {
        printf ("ERROR: Couldn't open graphics.library\n");
       }

       CloseLibrary ((struct Library *)IntuitionBase);
      } // OpenLib(intuition
      else
      {
       printf ("ERROR: Couldn't open intuition.library\n");
      }

      CloseDevice((struct IORequest *)timerio);
     } // OpenDevice
     else
     {
      printf("ERROR: Unable to open timer.device\n");
     }

     DeleteStdIO((struct IOStdReq *)timerio);
    } // if(CreateStdIO
    else
    {
     printf("ERROR: Unable to create timer stdio\n");
    }

    DeletePort(timermp);
   } // if(CreatePort
   else
   {
    printf ("ERROR: Unable to create timer msgport\n");
   }

   FreeArgs(rda);
  }
  else
  {
   printf("Usage: %s %s\n",argv[0],rdatemplate);
  }
}




void TransWorld(Point3D in[],TQAVTexture out[],
                int cnt,float wx,float wy,float wz,float z)
{
        int		i;
        float		cosx,sinx,cosy,siny,cosz,sinz;
        float           x1,y1,z1,h1;

	cosx = cos(wx);
	sinx = sin(wx);

	cosy = cos(wy);
	siny = sin(wy);

	cosz = cos(wz);
	sinz = sin(wz);

	 for (i=0;i<cnt;i++)                /* Index of In  Polygons */
	 {
		/* rotate in Y  */

                x1              = in[i].x;
                h1      =       z1      = in[i].z;

                if(wy != 0.0)
                {
			z1	=	siny*x1 + cosy*z1;
			x1      	=	cosy*x1 -siny*h1;
                }

		y1      =       h1	=	in[i].y;

		/* rotate in X  */

                if(wx != 0.0)
                {
			y1	=	cosx*y1 + sinx*z1;
			z1	=	-(sinx*h1) + cosx*z1;
                }

		/* rotate in Z  */

	        h1	=	x1;

                if(wz != 0.0)
                {
			x1	=	cosz*x1 + sinz*y1;
			y1	=	-sinz*h1 + cosz*y1;
                }

		/* move */

		z1      *= (1.0f/32767.0f);
                z1      +=       (float)z;

		out[i].x = ((x1*FOCUS)/(z1)) + WINX/2;
		out[i].y = ((y1*FOCUS)/(z1)) + WINY/2;
                out[i].z = out[i].invW = z1;

		if(z1 != 0.0) out[i].invW =  1.0f / (z1);

		out[i].uOverW = (float)(in[i].u - 1)/( (float)(128)) /out[i].z;
		out[i].vOverW  = (float)(in[i].v - 1)/( (float)(128)) /out[i].z;

		out[i].a = in[i].a;
		out[i].r = out[i].ks_r =out[i].kd_r = in[i].r;
		out[i].g = out[i].ks_g =out[i].kd_g = in[i].g;
		out[i].b = out[i].ks_b =out[i].kd_b = in[i].b;

	 }

}


void RenderWorld(TQAVTexture in[],int cnt)
{
        int		i;
		 TQAVGouraud  v;

	WaitBOVP(&v3d->ViewPort);

#if 0
	if(! SafeToWrite[1-BufNum])
		while(! GetMsg(sbuf[1-BufNum]->sb_DBufInfo->dbi_SafeMessage.mn_ReplyPort))
			Wait(1L<<((sbuf[1-BufNum]->sb_DBufInfo->dbi_SafeMessage.mn_ReplyPort->mp_SigBit)));
	SafeToWrite[BufNum] = TRUE;
#endif

        {

	   QARenderStart(drawContext,0L,0L);


	if(bitmap && InfoFlag)
	{
		 v.x = 0.0; v.y = 0;  v.z = 1.0;
		 v.a = 1.0; v.r = 1.0; v.g = v.b = 0.0;
		 QASetInt(drawContext,kQATag_ZFunction,kQAZFunction_None);
		 QADrawBitmap(drawContext,&v,bitmap);
		 if(ZBufFlag) QASetInt(drawContext,kQATag_ZFunction,kQAZFunction_LE);
	}

	 if(ZBufFlag)
	 {

			if(TextureFlag)
			{
	 		 QADrawVTexture(drawContext,cnt,kQAVertexMode_Tri,in,0L);
			}
			else
			{
			    for (i=0;i<cnt;i+=3)                /* Index of In Triangles */
			    {
		 		 QADrawTriGouraud(drawContext,(struct TQAVGouraud *)&in[i],(struct TQAVGouraud *)&in[i+1],(struct TQAVGouraud *)&in[i+2],kQATriFlags_None);
			    }
			}
	 }
	 else
	 {
	    for (i=0;i<cnt;i+=3)                /* Index of In Triangles */
	    {

		        //      if zbuffering disabled, only draw visible triangle (area normal to viewer)
                if(( (in[i+1].x-in[i].x) * (in[i+2].y-in[i+1].y) - (in[i+2].x-in[i+1].x) * (in[i+1].y-in[i].y)) < 0.0)
		{

			if(TextureFlag)
			{
	 		 QADrawTriTexture(drawContext,&in[i],&in[i+1],&in[i+2],kQATriFlags_None);
			}
			else
			{
	 		 QADrawTriGouraud(drawContext,(struct TQAVGouraud *)&in[i],(struct TQAVGouraud *)&in[i+1],(struct TQAVGouraud *)&in[i+2],kQATriFlags_None);
			}
		}
	    }
	};
#if 0
	   if(texbitmap)
		{

		 v.x = v.y = 0.0; v.z = 1.0;
		 v.a = 1.0; v.r = 1.0; v.g = v.b = 0.0;
		 QASetInt(drawContext,kQATag_ZFunction,kQAZFunction_None);
		 QADrawBitmap(drawContext,&v,texbitmap);

		 if(ZBufFlag) QASetInt(drawContext,kQATag_ZFunction,kQAZFunction_LE);
		}
#endif



	   QASync(drawContext);

//		SafeToWrite[1-BufNum] = FALSE;

	   QARenderEnd(drawContext,0L);
        }

#if 0
	if(! SafeToChange[1-BufNum])
		while(! GetMsg(sbuf[1-BufNum]->sb_DBufInfo->dbi_DispMessage.mn_ReplyPort))
			Wait(1L<<((sbuf[1-BufNum]->sb_DBufInfo->dbi_DispMessage.mn_ReplyPort->mp_SigBit)));
	SafeToChange[1-BufNum] = TRUE;
#endif

        BufNum++;
        BufNum&=1;

	ChangeScreenBuffer(v3d,sbuf[BufNum]);

//	SafeToChange[BufNum] = FALSE;
}


